function HA = linkzoom(varargin)
% LINKZOOM Links the zoom or pan for 2-D axes.
%
%   Syntax:
%          linkzoom
%          linkzoom(AS)
%          linkzoom(AH,...)
%     HA = linkzoom(...);
%
%   Input:
%     AS - String specifying the axes to be linked. Must be a combination
%          of 'x', 'y' and 'z' or 'off' to eliminate the link  (see note
%          below).
%          DEFAULT = 'xy' (both axes are linked)
%     AH - Handles of axes to be linked with/without handle of figures or
%          uipanels to search for them (ignores the colorbars).
%          DEFAULT: gcf (links all the axes from the current figure)
%
%   Output:
%     HA - Handles of the axes (see note below).
%
%   Description:
%     MATLAB's LINKAXES syncronizes the axes limits. This function linkes
%     the zoom/pan of several axes with different axis limits. That is,
%     applies the same zooming/pannig in all the axes without taking care
%     of the axes limits, only on the axes areas.
%
%     3 dimensional MATLABs ZOOM changes the camera view instead of axes
%     limits. For this reason, by now the program works only on
%     2-dimensional plots.
%
%   Note:
%     * To reset (for example) the zoom limits in all axes use: 
%       >> for k = 1:length(HA), zoom(HA(k),'reset'), end
%     * By now, MATLABs ZOOM does not work for z-axis/3-dimensional
%       zooming, and neither this function.
%
%   Example:
%  
%     % Links subplots on current figure:
%     figure(1)
%     t = 1:100;
%     subplot(211)
%      loglog(t,cos(t)+10,t,t.^2/10000+10)
%      axis tight
%     subplot(212)
%      surf(peaks), shading interp, view(2), colormap(gray), colorbar
%      axis tight
%     linkzoom
%      
%     % Links x-axes in different figures:
%     figure(2)
%      plot(t,sin(t))
%      axis tight
%     figure(3)
%     h1 = subplot(211);
%      plot(t,3*sin(t))
%      axis tight
%     h2 = subplot(212);
%      surf(peaks), shading interp, view(2), colormap(gray), colorbar
%      axis tight
%     linkzoom([2 3],'x')
%     
%     % Links specified axes:
%     figure(4)
%      surf(peaks), shading interp, view(2), colormap(gray), colorbar
%      axis tight
%     linkzoom([4 h2]) % this excludes h2 from previous link   
%
%   See also: LINKAXES, LINKPROP, ZOOM and PAN.

%   Copyright 2008 Carlos Adrian Vargas Aguilera
%   $Revision: 1.0 $  $Date: 2008/09/11 14:30:00 $

%   Written by
%   M.S. Carlos Adrian Vargas Aguilera
%   Physical Oceanography PhD candidate
%   CICESE 
%   Mexico, 2008
%   nubeobscura@hotmail.com
%
%   Download from:
%   http://www.mathworks.com/matlabcentral/fileexchange/loadAuthor.do?objec
%   tType=author&objectId=1093874

% Parameters:
appname = 'linkzoomdata'; % Name of the application data.
HA      = [];             % Default output.

% MAIN:
if ~ (nargin==2 && isstruct(varargin{2})) 
 % LINKZOOM call from window command or m-file:
 
 % Checks inputs and sets defaults:
 if ~nargin
  AH = gcf;
  AS = 'xyz';
 end
 if nargin==1
  if ischar(varargin{1})
   AS = varargin{1};
   AH = gcf;
  else
   AH = varargin{1};
   AS = 'xyz';
  end
 end
 if nargin==2
  AH = varargin{1};
  AS = varargin{2};
 end
 if nargin>2
  error('CVARGAS:linkzoom:TooManyInputs', ...
   'Maximum 2 inputs are required.')
 end  

 % Checks handles:
 ih = ishandle(AH);
 if ~all(ih)
  if ~sum(ih)
   error('CVARGAS:linkzoom:IncorrectHandleInput', ...
     'Input handle must be from figures, uipanels or axes.')
  end
  warning('CVARGAS:linkzoom:IncorrectHandleInput', ...
     'Some input handles were ignored.')
  AH = AH(ih);
 end
 % Searches for axes handles in figures and uipanels:
 HA = [];
 for k = 1:length(AH)
  switch get(AH(k),'type')
   case 'figure'
    ha = findobj(AH(k),'-depth',1,'Type','axes','-not','Tag','Colorbar');
   case 'uipanel'
    ha = findobj(AH(k),'-depth',1,'Type','axes','-not','Tag','Colorbar');
   case 'axes'
    ha = AH(k);
   otherwise
    error('CVARGAS:linkzoom:IncorrectHandleInput', ...
     'Input handle must be from figures, uipanels or axes.')
  end
  HA = [HA; ha(:)];
 end
 
 % Number of axes:
 na = length(HA);
 
 % Checks strings:
 as = lower(AS);
 switch as
  case 'x',   % continue
  case 'xy',  % continue
  case 'yx',  % continue
  case 'xyz', % continue
  case 'xzy', % continue
  case 'yxz', % continue
  case 'yzx', % continue
  case 'y',   % continue
  case 'yz',  % continue
  case 'zy',  % continue
  case 'z',   % continue
  case 'off'  % cleares linking:
   for k = 1:na
    if isappdata(HA(k),appname)
     data = getappdata(HA(k),appname);
     rmappdata(HA(k),appname)
     if isstruct(data) || isfield(data,'ah')
      ind          = (data.ah==HA(k)) | ~ishandle(data.ah);
      data.ah(ind) = [];
      for l = 1:length(data.ah)
       setappdata(data.ah(l),appname,data)
      end
     end
    end
    hz = zoom(HA(k));  
    hp = pan(ancestor(HA(k),{'figure','uipanel'},'toplevel'));
    set(hz,'ActionPostCallback',[]) 
    set(hp,'ActionPostCallback',[])
   end
   if ~nargout, clear HA, end
   return
  otherwise
    error('CVARGAS:linkzoom:IncorrectStringInput', ...
    ['String axes input must be a combination of ''x'', ''y'' and ''z'''...
     'or ''off''.'])
 end 

 % Removes axes from previous links:
 for k = 1:na
  if isappdata(HA(k),appname)
   data = getappdata(HA(k),appname);
   rmappdata(HA(k),appname)
   if isstruct(data) || isfield(data,'ah')
    ind          = (data.ah==HA(k)) | ~ishandle(data.ah);
    data.ah(ind) = [];
    for l = 1:length(data.ah)
     setappdata(data.ah(l),appname,data)
    end
   end
  end
 end 

 % Sets and saves application data:
 data     = [];
 data.ah  = HA(:);
 data.as  = as;
 data.ap  = false;
 for k = 1:na 
  % Resets limits zoom for double clicks in any axes undoes zooming.
  zoom(data.ah(k),'reset')
 end
 for k = 1:na
  setappdata(HA(k),appname,data);
  hz = zoom(HA(k));
  hp = pan(ancestor(HA(k),{'figure','uipanel'},'toplevel'));
  set(hz,'ActionPostCallback',@linkzoom) 
  set(hp,'ActionPostCallback',@linkzoom)
  set(hz,'ActionPreCallback' ,@linkzoom) 
  set(hp,'ActionPreCallback' ,@linkzoom)
 end

else
 
 % LINKZOOM from ZOOM or PAN function:
 try
  if ~isfield(varargin{2},'Axes') || ~isscalar(varargin{2}.Axes)
   if ~nargout, clear HA, end
   return
  end
 
  % Gets application data:
  ahc  = varargin{2}.Axes;
  data = getappdata(ahc,appname);
  if ~isstruct(data) && ~isfield(data,'ah')
   if ~nargout, clear HA, end
   return
  end
  
  % Ignores deleted axes:
  data.ah(~ishandle(data.ah)) = [];
  
  % Number of axes:
  na = length(data.ah);
  
  % Finds out if it is a pre or poscallback or double click:
  doubleclick = strcmp(get(gcf,'SelectionType'),'open');
  if ~data.ap && ~doubleclick
   % Precallback:
   data.ap = true;
   for k = 1:na
    setappdata(data.ah(k),appname,data);
   end
   if ~nargout, clear HA, end
   return
  else
   % Postcallback:
   data.ap = false;
  end

  % Probably here goes a DRAWNOW but it seems to work properly without it
  % and besides, I keep getting an error with figure WindowButtonDownFcn.
  
  % Gets axes limits:
  iothers = (data.ah~=ahc);
  range   = 1:na;
  limpre  = [zeros(na,5) ones(na,1)]; 
  for k = range
   limpre(k,:) = [getappdata(data.ah(k),'zoom_zoomOrigAxesLimits') ...
                  get(data.ah(k),'ZLim')];
  end
  limfix = limpre(~iothers,:);
  
  % Zooms the X-axis:
  if ismember('x',data.as)
   limnew  = get(ahc,'XLim');
   if strcmp(get(ahc,'XScale'),'log');
    limnew = real(log10(limnew));
   end
   for k = range(iothers)
    limfit = interp1(limfix(1:2),limpre(k,1:2),limnew,'linear','extrap');
    if strcmp(get(data.ah(k),'XScale'),'log');
     limfit = 10.^limfit;
    end
    set(data.ah(k),'XLim',limfit)
   end
  end
  
  % Zooms the Y-axis:
  if ismember('y',data.as)
   limnew  = get(ahc,'YLim');
   if strcmp(get(ahc,'YScale'),'log');
    limnew = real(log10(limnew));
   end
   for k = range(iothers)
    limfit = interp1(limfix(3:4),limpre(k,3:4),limnew,'linear','extrap');
    if strcmp(get(data.ah(k),'YScale'),'log');
     limfit = 10.^limfit;
    end
    set(data.ah(k),'YLim',limfit)
   end
  end
  
  % Zooms the Z-axis:
  if ismember('z',data.as)
   limnew  = get(ahc,'ZLim');
   if strcmp(get(ahc,'ZScale'),'log');
    limnew = real(log10(limnew));
   end
   for k = range(iothers)
    limfit = interp1(limfix(5:6),limpre(k,5:6),limnew,'linear','extrap');
    if strcmp(get(data.ah(k),'ZScale'),'log');
     limfit = 10.^limfit;
    end
    set(data.ah(k),'ZLim',limfit)
   end
  end
  
  % Updates data:
  for k = range
   setappdata(data.ah(k),appname,data);
  end

 catch
  % Do not sets an error only displays it:
   disp(lasterr)
 end
end

% Outputs?:
if ~nargout, clear HA, end